@@ -16,12 +16,13 @@ import matplotlib.pyplot as plt
 
 from lxml import etree
 from PIL import Image, ImageDraw, ImageFont
-import re
+
 
 def test_ini():
     for attr in dir(DWProducts):
         print(getattr(DWProducts, attr))
 
+
 class DWConfig:
 
     _config_file = 'WaterDetect.ini'
@@ -44,7 +45,8 @@ class DWConfig:
                  'detectwatercluster': 'maxmndwi',
                  'clustering_bands': "[['ndwi', 'Nir']]",
                  'graphs_bands': "[['mbwi', 'mndwi'], ['ndwi', 'mbwi']]",
-                 'plot_ts': 'False'
+                 'plot_ts': 'False',
+                 'calc_glint': 'True'
                  }
 
     _units = {'turb-dogliotti': 'FNU',
@@ -109,6 +111,30 @@ class DWConfig:
     def pdf_reports(self):
         return self.get_option('General', 'pdf_reports', evaluate=True)
 
+    @property
+    def calc_glint(self):
+        return self.get_option('General', 'calc_glint', evaluate=True)
+
+    @property
+    def glint_mode(self):
+        return self.get_option('General', 'glint_mode', evaluate=True)
+
+    @property
+    def min_glint_multiplier(self):
+        return self.get_option('General', 'min_glint_multiplier', evaluate=True)
+
+    @property
+    def pdf_resolution(self):
+        return self.get_option('General', 'pdf_resolution', evaluate=True)
+
+    @property
+    def pekel_water(self):
+        return self.get_option('General', 'pekel_water', evaluate=True)
+
+    @property
+    def pekel_accuracy(self):
+        return self.get_option('General', 'pekel_accuracy', evaluate=True)
+
     @property
     def save_indices(self):
         return self.get_option('General', 'save_indices', evaluate=True)
@@ -197,6 +223,10 @@ class DWConfig:
     def train_size(self):
         return self.get_option('Clustering', 'train_size', evaluate=True)
 
+    @property
+    def regularization(self):
+        return self.get_option('Clustering', 'regularization', evaluate=True)
+
     @property
     def min_train_size(self):
         return self.get_option('Clustering', 'min_train_size',evaluate=True)
@@ -291,6 +321,8 @@ class DWConfig:
             section_name = 'LandsatMasks'
         elif product == 'S2_THEIA':
             section_name = 'TheiaMasks'
+        elif product == 'S2_S2COR':
+            section_name = 'S2CORMasks'
         else:
             section_name = None
 
@@ -356,33 +388,47 @@ class DWutils:
         return [i for i in input_folder.iterdir() if i.is_dir()]
 
     @staticmethod
-    def calc_normalized_difference(img1, img2, mask=None):
+    def calc_normalized_difference(img1, img2, mask=None, compress_cte=0.02):
         """
         Calc the normalized difference of given arrays (img1 - img2)/(img1 + img2).
         Updates the mask if any invalid numbers (ex. np.inf or np.nan) are encountered
         :param img1: first array
         :param img2: second array
         :param mask: initial mask, that will be updated
+        :param compress_cte: amount of index compression. The greater, the more the index will be compressed towards 0
         :return: nd array filled with -9999 in the mask and the mask itself
         """
 
         # changement for negative SRE scenes
-        if mask is not None:
-            min_cte = np.min([np.min(img1[~mask]), np.min(img2[~mask])])
-        else:
-            min_cte = np.min([np.min(img1), np.min(img2)])
+        # ##### UPDATED ON 01/04/2021
 
-        if min_cte <= 0:
-            min_cte = -min_cte + 0.001
-        else:
-            min_cte = 0
+        # create a minimum array
+        min_values = np.where(img1 < img2, img1, img2)
 
-        nd = ((img1+min_cte)-(img2+min_cte)) / ((img1+min_cte) + (img2+min_cte))
+        # then create the to_add matrix (min values turned into positive + epsilon)
+        min_values = np.where(min_values <= 0, -min_values + 0.001, 0) + compress_cte
+
+        nd = ((img1 + min_values) - (img2 + min_values)) / ((img1 + min_values) + (img2 + min_values))
+
+        # # VERSION WITH JUST 1 CONSTANT
+        # if mask is not None:
+        #     min_cte = np.min([np.min(img1[~mask]), np.min(img2[~mask])])
+        # else:
+        #     min_cte = np.min([np.min(img1), np.min(img2)])
+        #
+        # print(f'Correcting negative values by {min_cte}')
+        #
+        # if min_cte <= 0:
+        #     min_cte = -min_cte + 0.001
+        # else:
+        #     min_cte = 0
+        #
+        # nd = ((img1+min_cte)-(img2+min_cte)) / ((img1+min_cte) + (img2+min_cte))
 
         # if any of the bands is set to zero in the pixel, makes a small shift upwards, as proposed by olivier hagole
         # https://github.com/olivierhagolle/modified_NDVI
         # nd = np.where((img1 > 0) & (img2 > 0), (img1-img2) / (img1 + img2), np.nan)
-                      # (img1+0.005-img2-0.005) / (img1+0.005 + img2+0.005))
+        # (img1+0.005-img2-0.005) / (img1+0.005 + img2+0.005))
 
         # nd = np.where((img1 <= 0) & (img2 <= 0), np.nan, (img1-img2) / (img1 + img2))
 
@@ -397,7 +443,7 @@ class DWutils:
         nd[np.isinf(nd)] = 1
 
         # nd_mask = np.isinf(nd) | np.isnan(nd) | mask
-        nd_mask = np.isnan(nd) | mask
+        nd_mask = np.isnan(nd) | (mask if mask is not None else False)
 
         nd = np.ma.array(nd, mask=nd_mask, fill_value=-9999)
 
@@ -405,7 +451,7 @@ class DWutils:
 
     @staticmethod
     def rgb_burn_in(red, green, blue, burn_in_array, color=None, min_value=None, max_value=None, colormap='viridis',
-                    fade=1, uniform_distribution=False, no_data_value=-9999, valid_value=1):
+                    fade=1, uniform_distribution=False, no_data_value=-9999, valid_value=1, transp=0.0):
         """
         Burn in a mask or a specific parameter into an RGB image for visualization purposes.
         The burn_in_array will be copied where values are different from no_data_value.
@@ -420,13 +466,14 @@ class DWutils:
         :param no_data_value: Value to ne unconsidered
         :param color: Tuple of color (R, G, B) to be used in the burn in
         :param fade: Fade the RGB bands to emphasize the copied values
+        :param transp: Transparency to use in the mask (0=opaque 1=completely transparent)
         :return: RGB image bands
         """
 
         if color:
-            new_red = np.where(burn_in_array == valid_value, color[0], red*fade)
-            new_green = np.where(burn_in_array == valid_value, color[1], green*fade)
-            new_blue = np.where(burn_in_array == valid_value, color[2], blue*fade)
+            new_red = np.where(burn_in_array == valid_value, color[0] * (1 - transp) + red * (transp), red * fade)
+            new_green = np.where(burn_in_array == valid_value, color[1] * (1 - transp) + green * (transp), green * fade)
+            new_blue = np.where(burn_in_array == valid_value, color[2] * (1 - transp) + blue * (transp), blue * fade)
 
         else:
             # the mask is where the value equals no_data_value
@@ -631,6 +678,19 @@ class DWutils:
         print('Saving image: ' + filename)
         return
 
+    @staticmethod
+    def tif_2_pdf(tif_file, resolution=600, scale=2000):
+        """Convert a TIF image into a PDF given a resolution"""
+
+        pdf_file = tif_file[:-4] + '.pdf'
+
+        print(f'Creating pdf file: {pdf_file}')
+        translate = f'gdal_translate -outsize {resolution} 0 -ot Byte -scale 0 {scale} -of pdf ' \
+                    + tif_file + ' ' + pdf_file
+        os.system(translate)
+
+        return pdf_file
+
     # -----------------------------------------------
     @staticmethod
     def array2multiband(filename, array, geo_transform, projection, nodatavalue=0, dtype=None):
@@ -766,7 +826,7 @@ class DWutils:
         return
 
     @staticmethod
-    def create_composite(bands, folder_name, pdf=True):
+    def create_composite(bands, folder_name, pdf=True, resolution=600):
 
         # copy the RGB clipped bands to output directory
 
@@ -780,8 +840,8 @@ class DWutils:
                   red_band + ' ' + green_band + ' ' + blue_band)
 
         if pdf:
-            cmd = 'gdal_translate -of pdf -ot Byte -scale 0 2000 -outsize 600 0 ' + composite_base_name + '.vrt ' \
-                  + composite_base_name + '.pdf'
+            cmd = f'gdal_translate -of pdf -ot Byte -scale 0 2000 -outsize {resolution} 0 ' \
+                  + composite_base_name + '.vrt ' + composite_base_name + '.pdf'
             os.system(cmd)
 
         return composite_base_name
@@ -875,7 +935,6 @@ class DWutils:
 
         return gdal_mask
 
-#------------------
     @staticmethod
     def extract_angles_from_xml(xml):
         """
@@ -912,10 +971,11 @@ class DWutils:
         azimuth_angle = root.xpath('//AZIMUTH_ANGLE[@unit="deg"]/text()')
         # azimuth_angle[0] = sun_angle and then Mean_Viewing_Incidence_Angle_List for B2, B3....
 
+        # the first values in the list correspond to the sun angle
         SZA = np.deg2rad(float(zenith_angle[0])) #radian
         SazA = float(azimuth_angle[0])
 
-        # Remove first element for both lists
+        # Remove sun angles (first element) from both lists
         zenith_angle.pop(0)
         azimuth_angle.pop(0)
 
@@ -934,6 +994,18 @@ class DWutils:
 
         return g
 
+    @staticmethod
+    def write_pdf(filename, text, size=(300, 50), position=(5, 5), font_color=(0, 0, 0)):
+        out = Image.new("RGB", size, (255, 255, 255))
+        draw = ImageDraw.Draw(out)
+
+        draw.multiline_text(position, text, fill=font_color, anchor=None, spacing=10, align="center")
+
+        out.save(filename)
+        out.close()
+
+        return filename
+
     @staticmethod
     def create_glint_pdf(xml, name_img, output_folder, g, pdf_merger):
         """
@@ -955,7 +1027,7 @@ class DWutils:
         # get a drawing context
         d = ImageDraw.Draw(out)
         # font size
-        font = ImageFont.truetype("arial.ttf", 16)
+        font = ImageFont.truetype("DejaVuSans.ttf", 16)
 
         # Test about glint values
         print("---------------------------")
@@ -993,7 +1065,6 @@ class DWutils:
             # Add to the main pdf
             pdf_merger.append(filename + '.pdf')
 
-
     @staticmethod
     def remove_negatives(bands, mask=None, negative_values='mask'):
         """
