@@ -1,81 +1,25 @@
 # #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-
 from waterdetect.InputOutput import DWSaver, DWLoader
-from waterdetect.Common import DWConfig, DWutils, gdal
+from waterdetect.Common import DWConfig, DWutils
 from waterdetect.Image import DWImageClustering
-from pathlib import Path
+from waterdetect.Glint import DWGlintProcessor
+from waterdetect import jaccard_score, gdal
+from waterdetect import __version__ as wd_version
+
 import numpy as np
 from PyPDF2 import PdfFileMerger
 from sklearn.preprocessing import MinMaxScaler, RobustScaler
-import argparse
 import os
 
-
 """
 Author: Mauricio Cordeiro
 """
 
 
-def main():
-    """
-    The main function is just a wrapper to create a entry point script called waterdetect.
-    With the package installed you can just call waterdetect -h in the command prompt to see the options.
-    """
-    parser = argparse.ArgumentParser(description='The waterdetect is a high speed water detection algorithm for sate'
-                                                 'llite images. It will loop through all images available in the input '
-                                                 'folder and write results for every combination specified in the'
-                                                 ' .ini file to the output folder. It can also run for single images '
-                                                 'from Python console or Jupyter notebook. Refer to the online'
-                                                 'documentation ',
-                                     epilog="To copy the package's default .ini file into the current directory, type:"
-                                            ' `waterdetect -GC .` without other arguments and it will copy  '
-                                            'WaterDetect.ini into the current directory.')
-
-    parser.add_argument("-GC", "--GetConfig", help="Copy the WaterDetect.ini from the package into the current "
-                                                   "directory and skips the processing. Once copied you can edit the "
-                                                   ".ini file and launch the waterdetect without -c option.",
-                        action="store_true")
-    parser.add_argument("-i", "--input", help="The products input folder. Required.", required=False, type=str)
-    parser.add_argument("-o", "--out", help="Output directory. Required.", required=False, type=str)
-    parser.add_argument("-s", "--shp", help="SHP file. Optional.", type=str)
-    parser.add_argument("-p", "--product", help='The product to be processed (S2_THEIA, L8_USGS, S2_L1C or S2_S2COR)',
-                        default='S2_THEIA', type=str)
-    parser.add_argument('-c', '--config', help='Configuration .ini file. If not specified WaterDetect.ini '
-                                               'from current dir and used as default', type=str)
-
-    # product type (theia, sen2cor, landsat, etc.)
-    # optional shape file
-    # generate graphics (boolean)
-    # name of config file with the bands-list for detecting, saving graphics, etc. If not specified, use default name
-    #   if clip MIR or not, number of pixels to plot in graph, number of clusters, max pixels to process, etc.
-    # name of the configuration .ini file (optional, default is WaterDetect.ini in the same folder
-
-    args = parser.parse_args()
-
-    # If GetConfig option, just copy the WaterDetect.ini to the current working directory
-    if args.GetConfig:
-        src = Path(__file__).parent/'WaterDetect.ini'
-        dst = Path(os.getcwd())/'WaterDetect.ini'
-
-        print(f'Copying {src} into current dir.')
-        dst.write_text(src.read_text())
-        print(f'WaterDetect.ini copied into {dst.parent}.')
-
-    else:
-        if (args.input is None) or (args.out is None):
-            print('Please specify input and output folders (-i, -o)')
-
-        else:
-            DWWaterDetect.run_batch(input_folder=args.input, output_folder=args.out, shape_file=args.shp,
-                                    product=args.product, config_file=args.config)
-
-    return
-
-
 class DWWaterDetect:
 
-    def __init__(self, input_folder, output_folder, shape_file, product, config_file, single_mode=False):
+    def __init__(self, input_folder, output_folder, shape_file, product, config_file, pekel=None, single_mode=False):
 
         # Create the Configuration object.
         # It loads the configuration file (WaterDetect.ini) and holds all the defaults if missing parameters
@@ -89,6 +33,7 @@ class DWWaterDetect:
         self.saver = DWSaver(output_folder, product, self.loader.area_name)
 
         self.single_mode = single_mode
+        self.pekel = pekel
 
         return
 
@@ -144,7 +89,8 @@ class DWWaterDetect:
         :return: index array
         """
 
-        index, mask = DWutils.calc_normalized_difference(band1, band2, self.loader.invalid_mask)
+        index, mask = DWutils.calc_normalized_difference(band1, band2, self.loader.invalid_mask,
+                                                         compress_cte=self.config.regularization)
         self.loader.update_mask(mask)
 
         self.loader.raster_bands.update({index_name: index})
@@ -264,7 +210,7 @@ class DWWaterDetect:
 
     @classmethod
     def run_batch(cls, input_folder, output_folder, shape_file=None, product='S2_THEIA',
-                  config_file=None, post_callback=None):
+                  config_file=None, pekel=None, post_callback=None):
         """
         Run batch is intended for multi processing of various images and bands combinations.
         It Loops through all unzipped images in input folder, extract water pixels and save results to output folder
@@ -277,6 +223,7 @@ class DWWaterDetect:
                  shape_file=shape_file,
                  product=product,
                  config_file=config_file,
+                 pekel=pekel,
                  single_mode=False)
 
         wd._detect_water(post_callback=post_callback)
@@ -292,6 +239,8 @@ class DWWaterDetect:
         :return: None
         """
 
+        print(f'Starting WaterDetection version: {wd_version}')
+
         # initialize the detect water instance variable with None
         dw_image = None
 
@@ -312,16 +261,6 @@ class DWWaterDetect:
                                      self.saver.temp_dir)
                     self.saver.update_geo_transform(image.geo_transform, image.projection)
 
-                # create a composite R G B in the output folder
-                if self.config.create_composite or self.config.pdf_reports:
-                    composite_name = DWutils.create_composite(image.gdal_bands, self.saver.output_folder,
-                                                              self.config.pdf_reports)
-                else:
-                    composite_name = None
-
-                # Load necessary bands in memory as a dictionary of names (keys) and arrays (Values)
-                image.load_raster_bands(self.necessary_bands(include_rgb=False))
-
                 # load the masks specified in the config (internal masks for theia or landsat) and the external tif mask
                 image.load_masks(self.config.get_masks_list(image.product),
                                  self.config.external_mask,
@@ -330,13 +269,27 @@ class DWWaterDetect:
                                  self.config.mask_invalid_value)
 
                 # Test if there is enough valid pixels in the clipped images
-                if (np.count_nonzero(image.invalid_mask) / image.invalid_mask.size) > self.config.maximum_invalid:
-                    print('Not enough valid pixels in the image area')
+                invalid_pct = np.count_nonzero(image.invalid_mask) / image.invalid_mask.size
+                if invalid_pct > self.config.maximum_invalid:
+                    print(f'Invalid pixels ({invalid_pct}) > maximum ({self.config.maximum_invalid}). Skipping image')
                     continue
+                else:
+                    print(f'Invalid pixels ({invalid_pct}) < maximum ({self.config.maximum_invalid}).')
+
+                # Load necessary bands in memory as a dictionary of names (keys) and arrays (Values)
+                image.load_raster_bands(self.necessary_bands(include_rgb=False))
 
                 # calc the necessary indices and update the image's mask
                 self.calc_indexes(image, indexes_list=['mndwi', 'ndwi', 'mbwi'], save_index=self.config.save_indices)
 
+                # create a composite R G B in the output folder
+                if self.config.create_composite or self.config.pdf_reports:
+                    composite_name = DWutils.create_composite(image.gdal_bands, self.saver.output_folder,
+                                                              self.config.pdf_reports, self.config.pdf_resolution)
+
+                else:
+                    composite_name = None
+
                 # if the method is average_results, the loop through bands_combinations will be done in DWImage module
                 if self.config.average_results:
                     print('Calculating water mask considering the average of combinations.')
@@ -355,8 +308,8 @@ class DWWaterDetect:
                         print('Calculating clusters for the following combination of bands:')
                         print(band_combination)
 
-                        dw_image = self.create_mask_report(image, band_combination, composite_name, pdf_merger,
-                                                           post_callback)
+                        dw_image = self.create_mask_report(image, band_combination, composite_name,
+                                                           pdf_merger, post_callback)
 
                     except Exception as err:
                         print('**** ERROR DURING CLUSTERING ****')
@@ -378,24 +331,66 @@ class DWWaterDetect:
         self.dw_image = dw_image
         return dw_image
 
+    def test_pekel(self, image, dw_image, pdf_merger_image):
+
+        water_mask = dw_image.water_mask.copy()
+        pekel_mask = gdal.Open(self.pekel).ReadAsArray(buf_xsize=image.x_size,
+                                                       buf_ysize=image.y_size)
+
+        # join invalid mask with pekels invalid
+        invalid_mask = image.invalid_mask.astype('bool') | (pekel_mask == 255)
+
+        pekel_mask = pekel_mask[~invalid_mask]
+        water_mask = water_mask[~invalid_mask]
+
+        # accuracy_score(pekel_mask > pekel_threshold, water_mask == 1)
+        result = jaccard_score(pekel_mask > self.config.pekel_water, water_mask == 1) * 100
+
+        # save result to the pdf_merger
+        pdf_name = os.path.join(self.saver.output_folder, 'pekel_test.pdf')
+
+        # write the resulting text
+        if result > self.config.pekel_accuracy:
+            text = f'PEKEL TEST - OK \nAccuracy Threshold: {self.config.pekel_accuracy}%\nJaccard Index = {result:.3f}%'
+            color = (0, 0, 0)
+        else:
+            text = f'*** PEKEL TEST FAILED *** \nAccuracy Threshold: {self.config.pekel_accuracy}%\n' \
+                   f'Jaccard Index = {result:.3f}%'
+            color = (220, 0, 0)
+
+        pdf_merger_image.append(DWutils.write_pdf(pdf_name, text, size=(300, 100), position=(50, 15), font_color=color))
+        return result
+
     def create_mask_report(self, image, band_combination, composite_name, pdf_merger, post_callback):
         # if pdf_reports, create a FileMerger for this specific band combination
         if self.config.pdf_reports & (pdf_merger is not None):
             pdf_merger_image = PdfFileMerger()
             pdf_merger_image.append(composite_name + '.pdf')
+            # pdf_merger_image.append(invalid_mask_name)
         else:
             pdf_merger_image = None
 
-        # create a dw_image object with the water mask and all the results
-        dw_image = self.create_water_mask(band_combination, pdf_merger_image)
-
         # calculate the sun glint rejection and add it to the pdf report
-        self.calc_glint(image, self.saver.output_folder, pdf_merger_image)
+        # the glint will be passed to the
+        if self.config.calc_glint:
+            glint_processor = DWGlintProcessor(image)
+            pdf_merger_image.append(glint_processor.save_heatmap(self.saver.output_folder))
+
+        else:
+            glint_processor = None
+            # self.calc_glint(image, self.saver.output_folder, pdf_merger_image)
+
+        # create a dw_image object with the water mask and all the results
+        dw_image = self.create_water_mask(band_combination, pdf_merger_image, glint_processor=glint_processor)
 
         # if there is a post processing callback, call it passing the mask and the pdf_merger_image
         if post_callback is not None:
             post_callback(self, dw_image=dw_image, pdf_merger=pdf_merger_image)
 
+        # Check the discrepancy of the water mask and pekel occurrence
+        if self.pekel:
+            self.test_pekel(image, dw_image, pdf_merger_image)
+
         # save the graphs
         if self.config.plot_graphs:
             self.save_graphs(dw_image, pdf_merger_image)
@@ -407,7 +402,8 @@ class DWWaterDetect:
         return dw_image
 
     # save the report and return the full path as posix
-    def save_report(self, report_name, pdf_merger, folder):
+    @staticmethod
+    def save_report(report_name, pdf_merger, folder):
 
         filename = folder.joinpath(report_name + '.pdf')
 
@@ -430,11 +426,10 @@ class DWWaterDetect:
         DWutils.plot_graphs(self.loader.raster_bands, self.config.graphs_bands, dw_image.cluster_matrix,
                             graph_basename, graph_title, self.loader.invalid_mask, 1000, pdf_merger_image)
 
-    def create_water_mask(self, band_combination, pdf_merger_image):
-
+    def create_water_mask(self, band_combination, pdf_merger_image, glint_processor=None):
         # create the clustering image
         dw_image = DWImageClustering(self.loader.raster_bands, band_combination,
-                                             self.loader.invalid_mask, self.config)
+                                     self.loader.invalid_mask, glint_processor, self.config)
         dw_image.run_detect_water()
 
         # save the water mask and the clustering results
@@ -444,31 +439,42 @@ class DWWaterDetect:
                               opt_relative_path=dw_image.product_name, dtype=gdal.GDT_Byte)
         # unload bands
 
-        # if there is a pdf to create, burn-in the mask into the RGB composite
+        # if there is a pdf to create, burn-in the mask into the RGB composit
+        # e
         # and append it to the image merger
         if pdf_merger_image:
             pdf_merger_image.append(self.create_rgb_burn_in_pdf(dw_image.product_name + '_water_mask',
-                                                                burn_in_array=dw_image.water_mask,
-                                                                color=(0, 0, 1),
+                                                                burn_in_arrays=dw_image.water_mask,
+                                                                colors=(0, 0, 1),
                                                                 fade=1,
                                                                 opt_relative_path=dw_image.product_name,
                                                                 valid_value=1))
 
+            pdf_merger_image.append(self.create_rgb_burn_in_pdf(dw_image.product_name + '_overlay',
+                                                                burn_in_arrays=[dw_image.water_mask,
+                                                                                self.loader.invalid_mask],
+                                                                colors=[(0, 0, 1), (1, 0, 0)],
+                                                                fade=1,
+                                                                opt_relative_path=dw_image.product_name,
+                                                                valid_value=1,
+                                                                transps=[0, 0.7]))
+
         return dw_image
 
-    def calc_glint(self, image, output_folder, pdf_merger_image):
-        """
-        Calculate the sun glint rejection using the angle Tetag between vectors pointing in the surface-to-satellite
-        and specular reflection directions
-        Also, checks if there are reports, then add the risk of glint to it.
-        """
-        xml = str(self.loader.metadata)
-        # check the path of the metadata file
-        DWutils.check_path(xml)
-        # extract angles from the metadata and make the glint calculation from it
-        glint = DWutils.extract_angles_from_xml(xml)
-        # create a pdf file that indicate if there is glint on the image and add it to the final pdf report
-        DWutils.create_glint_pdf(xml, self.loader.glint_name, output_folder, glint, pdf_merger_image)
+    # def calc_glint(self, image, output_folder, pdf_merger_image):
+    #     """
+    #     Calculate the sun glint rejection using the angle Tetag between vectors pointing in the surface-to-satellite
+    #     and specular reflection directions
+    #     Also, checks if there are reports, then add the risk of glint to it.
+    #     """
+    #     xml = str(self.loader.metadata)
+    #     # check the path of the metadata file
+    #     DWutils.check_path(xml)
+    #     # extract angles from the metadata and make the glint calculation from it
+    #     glint = DWutils.extract_angles_from_xml(xml)
+    #
+    #     # create a pdf file that indicate if there is glint on the image and add it to the final pdf report
+    #     DWutils.create_glint_pdf(xml, self.loader.glint_name, output_folder, glint, pdf_merger_image)
 
     def create_colorbar_pdf(self, product_name, colormap, min_value, max_value):
 
@@ -487,23 +493,48 @@ class DWWaterDetect:
 
         return filename.as_posix()
 
-    def create_rgb_burn_in_pdf(self, product_name, burn_in_array, color=None, min_value=None, max_value=None,
+    def create_rgb_burn_in_pdf(self, product_name, burn_in_arrays, colors=None, min_value=None, max_value=None,
                                fade=None, opt_relative_path=None, colormap='viridis', uniform_distribution=False,
-                               no_data_value=0, valid_value=1):
+                               no_data_value=0, valid_value=1, transps=None, bright=5.):
 
-        # create the RGB burn in image
-        red, green, blue = DWutils.rgb_burn_in(red=self.loader.raster_bands['Red'],
-                                               green=self.loader.raster_bands['Green'],
-                                               blue=self.loader.raster_bands['Blue'],
-                                               burn_in_array=burn_in_array,
-                                               color=color,
-                                               min_value=min_value,
-                                               max_value=max_value,
-                                               colormap=colormap,
-                                               fade=fade,
-                                               uniform_distribution=False,
-                                               no_data_value=no_data_value,
-                                               valid_value=valid_value)
+        red = self.loader.raster_bands['Red']*bright
+        green = self.loader.raster_bands['Green']*bright
+        blue = self.loader.raster_bands['Blue']*bright
+
+        # limit the maximum brightness to 1
+        red[red > 1] = 1
+        green[green > 1] = 1
+        blue[blue > 1] = 1
+
+        if isinstance(burn_in_arrays, list):
+            for burn_in_array, color, transp in zip(burn_in_arrays, colors, transps):
+                red, green, blue = DWutils.rgb_burn_in(red=red,
+                                                       green=green,
+                                                       blue=blue,
+                                                       burn_in_array=burn_in_array,
+                                                       color=color,
+                                                       min_value=min_value,
+                                                       max_value=max_value,
+                                                       colormap=colormap,
+                                                       fade=fade,
+                                                       uniform_distribution=uniform_distribution,
+                                                       no_data_value=no_data_value,
+                                                       valid_value=valid_value,
+                                                       transp=transp)
+        else:
+            # create the RGB burn in image
+            red, green, blue = DWutils.rgb_burn_in(red=red,
+                                                   green=green,
+                                                   blue=blue,
+                                                   burn_in_array=burn_in_arrays,
+                                                   color=colors,
+                                                   min_value=min_value,
+                                                   max_value=max_value,
+                                                   colormap=colormap,
+                                                   fade=fade,
+                                                   uniform_distribution=uniform_distribution,
+                                                   no_data_value=no_data_value,
+                                                   valid_value=valid_value)
 
         # save the RGB auxiliary tif and gets the full path filename
         filename = self.saver.save_rgb_array(red=red * 10000,
@@ -512,11 +543,12 @@ class DWWaterDetect:
                                              name=product_name+'_rgb',
                                              opt_relative_path=opt_relative_path)
 
-        new_filename = filename[:-4] + '.pdf'
-        translate = 'gdal_translate -outsize 600 0 -ot Byte -scale 0 2000 -of pdf ' + filename + ' ' + new_filename
-        os.system(translate)
+        pdf_filename = DWutils.tif_2_pdf(filename, self.config.pdf_resolution, scale=10000)
 
-        return new_filename
+        # remove the RGB auxiliary tif (too big to be kept)
+        os.remove(filename)
+
+        return pdf_filename
 
     def calc_indexes(self, image, indexes_list, save_index=False):
 
@@ -543,11 +575,7 @@ class DWWaterDetect:
             self.calc_awei(raster_bands, save_index=save_index)
 
         # update the final mask
+        print('Saving final MASK')
         self.saver.save_array(image.invalid_mask, image.current_image_name + '_invalid_mask', dtype=gdal.GDT_Byte)
 
         return
-
-# check if this file has been called as script
-if __name__ == '__main__':
-    main()
-
